/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggSQUISH PLAYER/CONVERTER SOURCE CODE. *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS SOURCE IS RESTRICTED  *
 * BY DISTRIBUTION TERMS THAT SHOULD BE INCLUDED WITH THIS SOURCE.  *
 * PLEASE READ THESE TERMS BEFORE USING OR DISTRIBUTING.            *
 *                                                                  *
 * THE OggSQUISH SOURCE CODE IS (C) COPYRIGHT 1994-1996             *
 * XIPHOPHORUS                                                      *
 *                                                                  *
 ********************************************************************

  file: linux.c
  function: Playback code for Linux and OSS
  author: Monty <xiphmont@mit.edu>
  modifications by: Monty
  last modification date: Oct 10 1996

 ********************************************************************/

/* TODO:

   play with small fragments/ better banking to give us the most time
   possible between blocking to process a frame */

/* Can we set a playback mode exactly matching the sample? */
/* If not, set the best we can, or use transforms and effects to force
   fit the playback to the hardware */

#if defined (__linux__) || defined (OPEN_SOUND_SYSTEM)

#include <sys/soundcard.h>
#include <sys/ioctl.h>

FILE *audioout=NULL;

extern int OGG_ess;
extern long OGG_polyphonic;
extern long OGG_mulaw;
extern long O_channels;
extern long O_rate;
extern long O_bits;
extern long O_bigendian;
extern long O_signed;

extern long I_channels;
extern long I_rate;
extern long I_bits;
extern long I_bigendian;
extern long I_signed;

extern int OGG_quality;
extern int OGG_time;

extern long OGG_begin;
extern long OGG_end;
extern long OGG_frames;

extern int OGG_in;  /* 0 OggSquish, 1 Raw, 2 WAVE, 3 AIFF[-C] */
extern int OGG_out; /* -1 playback, 0 OggSquish, 1 Raw, 2 WAVE, 3 AIFF[-C] */
extern int OGG_action; /* 0 encode, 1 decode, 2 play */

extern int X_channel;
extern int X_resample;
extern int X_volume;

void setplayback(char *outfilename){
  char buffer[160];
  int audio,temp,temp2;

  if(outfilename==NULL){
    /* Try to open /dev/dsp, /dev/dsp1, /dev/dsp2, /dev/audio */
    if((audioout=fopen("/dev/dsp","wb"))==NULL)
    if((audioout=fopen("/dev/dsp1","wb"))==NULL)
    if((audioout=fopen("/dev/dsp2","wb"))==NULL)
    audioout=fopen("/dev/audio","wb");
  }else
    audioout=fopen(outfilename,"wb");

  if(audioout==NULL)
    reporterror("Unable to open audio playback device.");

  audio=fileno(audioout);
  if(audio==-1)
    reporterror("Unable to open audio playback device");
    
  /* Set format */
  /* Go for exact first */
  switch(O_bits){
  case -8:
    temp=AFMT_MU_LAW;
    break;
  case 8:
    temp=AFMT_U8;
    O_signed=0;
    break;
  default:
    temp=AFMT_S16_LE;
    O_signed=-1;
    O_bigendian=OGG_LITTLEENDIAN;
    O_bits=16;
    break;
  }

  temp2=temp;
  if(ioctl(audio,SNDCTL_DSP_SETFMT,&temp)==-1 || temp!=temp2){
    /* Oops, no good...  Let's try a bunch of formats till we can set one.
       Libogg can do these conversions trivially */

    temp=AFMT_S16_LE;
    O_signed=-1;
    O_bigendian=OGG_LITTLEENDIAN;
    O_bits=16;
    temp2=temp;
    if(ioctl(audio,SNDCTL_DSP_SETFMT,&temp)==-1 || temp2!=temp){
      temp=AFMT_S16_BE;
      O_signed=-1;
      O_bigendian=OGG_BIGENDIAN;
      O_bits=16;
      temp2=temp;
      if(ioctl(audio,SNDCTL_DSP_SETFMT,&temp)==-1 || temp2!=temp){
	temp=AFMT_U8;
	O_signed=0;
	O_bigendian=OGG_LITTLEENDIAN;
	O_bits=8;
	temp2=temp;
	if(ioctl(audio,SNDCTL_DSP_SETFMT,&temp)==-1 || temp2!=temp)
	  /* Oh, boy...  Die miserably... */
	  reporterror("Unable to set any common playback format");
	else
	  report("Best available playback mode is 8 bit");
      }else
	report("Using 16 bit playback");
    }else
      report("Using 16 bit playback");
  }

  /* Well, OK, format is set... */

  /* Set channels */

  switch(O_channels){
  case 1:
    temp=0;
    if(ioctl(audio,SNDCTL_DSP_STEREO,&temp)==-1 || temp!=0)
      reporterror("Unable to set monophonic playback mode.");
    break;
  case 2:
    temp=1;
    if(ioctl(audio,SNDCTL_DSP_STEREO,&temp)==-1 || temp!=1){
      /* Ack... try to set mono */
      temp=0;
      if(ioctl(audio,SNDCTL_DSP_STEREO,&temp)==-1 || temp!=0)
	reporterror("Unable to set stereo or mono playback mode.");
      O_channels=1;
      X_channels=1;
      report("Stereo playback unavailable; using mono.");
    }
    break;
  default:
    temp=1;
    if(ioctl(audio,SNDCTL_DSP_STEREO,&temp)==-1 || temp!=1){
      /* Ack... try to set mono */
      temp=0;
      if(ioctl(audio,SNDCTL_DSP_STEREO,&temp)==-1 || temp!=1)
	reporterror("Unable to set stereo or mono playback mode.");
      O_channels=1;
      X_channels=1;
      report("Using mono playback");
    }else{
      report("Using stereo playback");
      O_channels=2;
      X_channels=2;
    }
    break;
  }
      
  temp=O_rate;
  if(ioctl(audio,SNDCTL_DSP_SPEED,&temp)==-1)
    reporterror("Error setting playback speed");
  
  if(fabs(temp-O_rate)/O_rate<.01)temp=O_rate;

  if(temp!=O_rate){
    sprintf(buffer,"Unable to set playback speed of %ldHz. Resampling to %ldHz.",
	    (long)O_rate,(long)temp);
    report(buffer);
    X_resample=O_rate=temp;
  }
}

void closeplayback(void){
  fclose(audioout);
}

void killplayback(void){
  int audio=fileno(audioout);
  if(audio==-1)return;
  ioctl(audio,SNDCTL_DSP_RESET);
  closeplayback();
}

void playback(void *buffer,long bytes, long bytesperframe){
  
  if(OGG_ess && O_bits==16){
    /* The ESS AudioDrive has a bigtime zero crossing bug; change
       the data into 15 nonnegative bits.  Playback is better
       (much less distortion) although we lose ~6dB SNR */

    long i,els;
    els=bytes>>1;
    for(i=0;i<els;i++)
      ((size16 *)buffer)[i]=(((size16 *)buffer)[i]>>1)+0x4000;
  }
  blocking_write(buffer,1,bytes,audioout);
  fflush(audioout);
}

#endif
