/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggSQUISH SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS SOURCE IS RESTRICTED  *
 * BY DISTRIBUTION TERMS THAT SHOULD BE INCLUDED WITH THIS SOURCE.  *
 * PLEASE READ THESE TERMS BEFORE USING OR DISTRIBUTING.            *
 *                                                                  *
 * THE OggSQUISH SOURCE CODE IS (C) COPYRIGHT 1994-1996             *
 * XIPHOPHORUS                                                      *
 *                                                                  *
 ********************************************************************

  file: registry.c
  function: Virtual machine module registration struct
  author: Monty <xiphmont@mit.edu>
  modifications by: Monty
  last modification date: Oct 15 1996

 ********************************************************************/

#include "A.h"
#include "B.h"
#include "C.h"
#include "D.h"
#include "E.h"
#include "F.h"
#include "J.h"
#include "P.h"
#include "S.h"
#include "W.h"

typedef struct mod {
  char *name;
  char *typelist;
  void (*create)(module *);
} mod;

/* B bitpipe
   P pipe
   V scalar variable
   M module
   A arith expression
   C character string
   L location (execution order)
   p,v,a,b,m,c,l same in context of a vararg list
*/

static mod registry[]={ 
  {"Aid","AAPAAAA",__oggc_Aid},   /* DSP input to pipe */
  {"Aib","AABAA",__oggc_Aib},     /* bitstream input to bpipe */
  {"Aod","AAPAAA",__oggc_Aod},    /* DSP output from pipe */
  {"Aob","AABA",__oggc_Aob},      /* bitstream output from bpipe */

  {"B","",NULL},                  /* Raw write backend */
  {"Bh","",NULL},                 /* Huffman backend */
  {"Bdpl","AMAAA",__oggc_Bdpl},   /* lossless max/val pair encoder (layered) */
  {"Bdplb","AAMAAA",__oggc_Bdplb},/* as above, layers sparse by ilog */
  {"Bdpc","AAMAAA",__oggc_Bdpc},  /* codeword backend encoder */
  {"Bxc","M",__oggc_Bxc},         /* commit changes to backend dictionary */
  {"Bxu","M",__oggc_Bxu},         /* undo dict changes to last commit */

  {"Covr","PAMB",__oggc_Covr},    /* flat vector output */
  {"Civrs","PAAAMB",__oggc_Civrs},/* signed flat vector input */
  {"Civr","PAAAMB",__oggc_Civr},  /* unsigned flat vector input */
  {"Conr","PB",__oggc_Conr},      /* flat negstream out */
  {"Cinr","PB",__oggc_Cinr},      /* flat negstream in */
  {"Conrl","PB",__oggc_Conrl},    /* As above, does not encode zeros */
  {"Cinrl","PB",__oggc_Cinrl},    /* As above */

#ifdef PRE_COMPAT
  {"Civta","PBAAAma",__oggc_Civta}, 
  {"Civt","PBAAAma",__oggc_Civt}, 
#endif

  {"Covtl","PBAm",__oggc_Covtl},   /* tree encoded vector out */
  {"Civtl","PBAAAm",__oggc_Civtl}, /* input */

  {"Covhl","PPBAM",__oggc_Covhl},  /* hybrid vector encoding output */
  {"Civhl","PPBAAAM",__oggc_Civhl},/* hybrid vector encoding input */

  {"Conc","PBAAMA",__oggc_Conc},   /* negstream codeword out */
  {"Cinc","PBAAMA",__oggc_Cinc},   /* negstream codeword in */
  {"Covc","PBM",__oggc_Covc},      /* codeworded vector output */
  {"Civc","PBAAM",__oggc_Civc},    /* codeworded vector input */

  {"Coa","AAB",__oggc_Coa},        /* scalar out */
  {"Cia","VAB",__oggc_Cia},        /* scalar in */

  {"Dca","PPVA",__oggc_Dca},       /* Generate an LPC coefficient set */
  {"Dr","PPPPAAP",__oggc_Dr},      /* LPC residue from input */
  {"Dp","PPPPP",__oggc_Dp},        /* repredict data from LPC residue */
  {"Dr32","PPPP",__oggc_Dr32},     /* As Dr, 32 int (limited dyn range) */
  {"Dp32","PPPP",__oggc_Dp32},     /* As Dp, 32 int (limited dyn range) */
  {"Dr64","PPPP",__oggc_Dr64},     /* As Dr, 64 int */
  {"Dp64","PPPP",__oggc_Dp64},     /* As Dp, 64 int */
  {"Ddf","PPP",__oggc_Ddf},        /* Forward delta */
  {"Ddi","PPP",__oggc_Ddi},        /* Reverse delta */

  {"Es","PPPA",__oggc_Es},         /* Vector split */
  {"Ez","PAAA",__oggc_Ez},         /* Generate vector */
  {"Ezo","PAAA",__oggc_Ezo},       /* Overwrite piece of vector with scalar */
  {"Ec","PPP",__oggc_Ec},          /* Concat vectors */
  {"Eq","PA",__oggc_Eq},           /* Quantize vector */
  {"Eqi","PA",__oggc_Eqi},         /* Reverse quantization */
  {"El","PP",__oggc_El},           /* Limit vector against ref vector */
  {"Edi","PPP",__oggc_Edi},        /* Apply drift scalar to vector */
  {"Eaa","PP",__oggc_Eaa},         /* Vector addition */
  {"Eas","PP",__oggc_Eas},         /* Vector subtraction */
  {"Ean","P",__oggc_Ean},          /* Vector negation */
  {"Eab","P",__oggc_Eab},          /* Vector absolute value */
  {"Eam","PAA",__oggc_Eam},        /* Vector multiply by scalar */
  {"EamF","PAA",__oggc_EamF},      /* Vector multiply by scalar (float) */
  {"Eav","PAV",__oggc_Eav},        /* Average elements of vector */
  {"Ecs","PAP",__oggc_Ecs},        /* Split one channel out of vector bundle */
  {"Ecr","PAP",__oggc_Ecr},        /* Replace channel of input vector */
  {"Est","PP",__oggc_Est},         /* polyphonic test analysis */
  {"Esa","PP",__oggc_Esa},         /* polyphonic channel mix application */

  {"Ff","PP",__oggc_Ff},           /* Forward dicrete real fft */
  {"Ffi","PP",__oggc_Ffi},         /* inverse fft */
  {"Fcq","PP",__oggc_Fcq},         /* forward real quarter wave DCT */
  {"Fcqi","PP",__oggc_Fcqi},       /* inverst DCT */
  {"Ff32","PP",__oggc_Ff32},       /* 32 bit int fft (very limited range)*/
  {"Ffi32","PP",__oggc_Ffi32},     /* inverse 32 bit dct */
  {"Fcq32","PP",__oggc_Fcq32},     /* 32 bit int fft (very limited range)*/
  {"Fcqi32","PP",__oggc_Fcqi32},   /* inverse 32 bit dct */
  {"Ffs","PPAA",__oggc_Ffs},       /* fft and scale */
  {"Ffis","PPAA",__oggc_Ffis},     /* scaled inverse fft */
  {"Fcqs","PPAA",__oggc_Fcqs},     /* scaled dct */
  {"Fcqis","PPAA",__oggc_Fcqis},   /* scaled inverse dct */
  {"Fn","P",__oggc_Fn},            /* Octave normalization */

  {"J","L",__oggc_J},              /* Jump */
  {"Jle","AAL",__oggc_Jle},        
  {"Jl","AAL",__oggc_Jl},
  {"Jge","AAL",__oggc_Jge},
  {"Jg","AAL",__oggc_Jg},
  {"Jne","AAL",__oggc_Jne},
  {"Je","AAL",__oggc_Je},
  {"Jz","AL",__oggc_Jz},
  {"Jnz","AL",__oggc_Jnz},
  {"Jwle","AALB",__oggc_Jle},      /* Jump <= and write decision bit */
  {"Jwl","AALB",__oggc_Jl},
  {"Jwge","AALB",__oggc_Jge},
  {"Jwg","AALB",__oggc_Jg},
  {"Jwne","AALB",__oggc_Jne},
  {"Jwe","AALB",__oggc_Je},
  {"Jwz","ALB",__oggc_Jz},
  {"Jwnz","ALB",__oggc_Jnz},
  {"Jr","LB",__oggc_Jr},           /* Jump if read bit == 1 */
  {"Jfor","VAAL",__oggc_Jfor},     /* For loop */

  {"Povhl","PAP",__oggc_Povhl},    /* Preprocess hybrid out */
  {"Povh","PAAP",__oggc_Povh},     /* Like above, lossy */
  {"Povh","PAAP",__oggc_Pivh},     /* postprocess, lossy */

  {"Sof","PPBAV",__oggc_Sof},        /* frame/termination detection */
  {"Sif","BAV",__oggc_Sif},          /* ...on input */
  {"Sofr","PPBAAAAAVV",__oggc_Sofr}, /* adaptive framing */
  {"Sifr","BAAVV",__oggc_Sifr},      /* ...on input */
  {"Shalt","AA",__oggc_Shalt},       /* Halt processing */
  {"Spm","PP",__oggc_Spm},           /* move pipe contents */
  {"Sbm","BB",__oggc_Sbm},           /* move bpipe contents */
  {"Spc","PP",__oggc_Spc},           /* copy pipe contents */
  {"Sbc","BB",__oggc_Sbc},           /* copy bpipe contents */
  {"Sba","BB",__oggc_Sba},           /* append bpipe contents */
  {"Sbt","BV",__oggc_Sbt},           /* test bpipe bits */
  {"Sbr","B",__oggc_Sbr},            /* reset bpipe */
  {"Spr","P",__oggc_Spr},            /* reset pipe */
  {"Spt","PV",__oggc_Spt},           /* test pipe contents length */
  {"Sv","AV",__oggc_Sv},             /* set variable */
  {"Spush","AP",__oggc_Spush},       /* push value into pipe */
  {"Spull","PV",__oggc_Spull},       /* pull value from pipe */

  {"Whl","PA",__oggc_Whl},    /* Window Hanning left */
  {"Whr","PA",__oggc_Whr},    /* Window Hanning right */
  {"Whli","PA",__oggc_Whli},  /* inverse Window Hanning left */
  {"Whri","PA",__oggc_Whri},  /* inverse Window Hanning right */

  {NULL,NULL,NULL}};

